package com.sun.xml.internal.bind.v2.runtime.unmarshaller;

import com.sun.xml.internal.bind.v2.runtime.JAXBContextImpl;
import javax.xml.transform.sax.TransformerHandler;
import org.xml.sax.SAXException;
import javax.xml.bind.annotation.DomHandler;
import javax.xml.transform.Result;

public class DomLoader<ResultT extends Result> extends Loader
{
    private final DomHandler<?, ResultT> dom;
    
    public DomLoader(DomHandler<?, ResultT> dom) {
        super(true);
        this.dom = dom;
    }
    
    @Override
    public void startElement(UnmarshallingContext.State state, TagName ea) throws SAXException {
        UnmarshallingContext context = state.getContext();
        if (state.getTarget() == null) {
            state.setTarget(new DomLoader.State(context));
        }
        DomLoader.State s = (DomLoader.State)state.getTarget();
        try {
            DomLoader.State.access$100(s, context, context.getNewlyDeclaredPrefixes());
            s.handler.startElement(ea.uri, ea.local, ea.getQname(), ea.atts);
        }
        catch (SAXException e) {
            context.handleError(e);
            throw e;
        }
    }
    
    @Override
    public void childElement(UnmarshallingContext.State state, TagName ea) throws SAXException {
        state.setLoader(this);
        DomLoader.State s = (DomLoader.State)state.getPrev().getTarget();
        ++s.depth;
        state.setTarget(s);
    }
    
    @Override
    public void text(UnmarshallingContext.State state, CharSequence text) throws SAXException {
        if (text.length() == 0) {
            return;
        }
        try {
            DomLoader.State s = (DomLoader.State)state.getTarget();
            s.handler.characters(text.toString().toCharArray(), 0, text.length());
        }
        catch (SAXException e) {
            state.getContext().handleError(e);
            throw e;
        }
        return;
    }
    
    @Override
    public void leaveElement(UnmarshallingContext.State state, TagName ea) throws SAXException {
        DomLoader.State s = (DomLoader.State)state.getTarget();
        UnmarshallingContext context = state.getContext();
        try {
            s.handler.endElement(ea.uri, ea.local, ea.getQname());
            DomLoader.State.access$300(s, context.getNewlyDeclaredPrefixes());
        }
        catch (SAXException e) {
            context.handleError(e);
            throw e;
        }
        int depth = s.depth - 1;
        s.depth = depth;
        if (depth == 0) {
            try {
                DomLoader.State.access$300(s, context.getAllDeclaredPrefixes());
                s.handler.endDocument();
            }
            catch (SAXException e) {
                context.handleError(e);
                throw e;
            }
            state.setTarget(s.getElement());
        }
    }
    
    private final class State
    {
        private TransformerHandler handler;
        private final ResultT result;
        int depth;
        
        public State(UnmarshallingContext context) throws SAXException {
            this.handler = null;
            this.depth = 1;
            this.handler = JAXBContextImpl.createTransformerHandler(context.getJAXBContext().disableSecurityProcessing);
            this.result = DomLoader.this.dom.createUnmarshaller(context);
            this.handler.setResult(this.result);
            try {
                this.handler.setDocumentLocator(context.getLocator());
                this.handler.startDocument();
                this.declarePrefixes(context, context.getAllDeclaredPrefixes());
            }
            catch (SAXException e) {
                context.handleError(e);
                throw e;
            }
        }
        
        public Object getElement() {
            return DomLoader.this.dom.getElement(this.result);
        }
        
        private void declarePrefixes(UnmarshallingContext context, String[] prefixes) throws SAXException {
            int i = prefixes.length - 1;
            while (i >= 0) {
                String nsUri = context.getNamespaceURI(prefixes[i]);
                if (nsUri == null) {
                    throw new IllegalStateException("prefix '" + prefixes[i] + "' isn't bound");
                }
                this.handler.startPrefixMapping(prefixes[i], nsUri);
                --i;
            }
        }
        
        private void undeclarePrefixes(String[] prefixes) throws SAXException {
            for (int i = prefixes.length - 1; i >= 0; --i) {
                this.handler.endPrefixMapping(prefixes[i]);
            }
        }
        
        static void access$100(DomLoader.State x0, UnmarshallingContext x1, String[] x2) throws SAXException {
            x0.declarePrefixes(x1, x2);
        }
        
        static void access$300(DomLoader.State x0, String[] x1) throws SAXException {
            x0.undeclarePrefixes(x1);
        }
    }
}
